from django.urls import path

from email_registration.views import email_registration_confirm, email_registration_form


urlpatterns = [
    path(
        "",
        email_registration_form,
        name="email_registration_form",
    ),
    path(
        "<str:code>/",
        email_registration_confirm,
        name="email_registration_confirm",
    ),
]



from django import forms
from django.contrib import messages
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import SetPasswordForm
from django.core.exceptions import FieldDoesNotExist
from django.shortcuts import redirect, render
from django.utils.crypto import get_random_string
from django.utils.translation import gettext as _, gettext_lazy
from django.views.decorators.http import require_POST

from email_registration.signals import password_set
from email_registration.utils import InvalidCode, decode, send_registration_mail


User = get_user_model()
USERNAME_FIELD = User.USERNAME_FIELD


class RegistrationForm(forms.Form):
    email = forms.EmailField(
        label=gettext_lazy("email address"),
        max_length=75,
        widget=forms.TextInput(
            attrs={
                "placeholder": gettext_lazy("email address"),
            }
        ),
    )

    def clean_email(self):
        email = self.cleaned_data.get("email")
        if email and User.objects.filter(email=email).exists():
            raise forms.ValidationError(
                _(
                    "This email address already exists as an account."
                    " Did you want to reset your password?"
                )
            )
        return email


@require_POST
def email_registration_form(request, form_class=RegistrationForm):
    # TODO unajaxify this view for the release?
    form = form_class(request.POST)

    if form.is_valid():
        email = form.cleaned_data["email"]
        send_registration_mail(email, request)

        return render(
            request,
            "registration/email_registration_sent.html",
            {
                "email": email,
            },
        )

    return render(
        request,
        "registration/email_registration_form.html",
        {
            "form": form,
        },
    )


def email_registration_confirm(
    request, code, max_age=3 * 86400, form_class=SetPasswordForm
):
    try:
        email, user = decode(code, max_age=max_age)
    except InvalidCode as exc:
        messages.error(request, "%s" % exc)
        return redirect("/")

    if not user:
        if User.objects.filter(email=email).exists():
            messages.error(
                request,
                _(
                    "This email address already exists as an account."
                    " Did you want to reset your password?"
                ),
            )
            return redirect("/")

        username_field = User._meta.get_field(USERNAME_FIELD)

        kwargs = {}
        if username_field.name == "email":
            kwargs["email"] = email
        else:
            username = email

            # If email exceeds max length of field set username to random
            # string
            max_length = username_field.max_length
            if len(username) > max_length:
                username = get_random_string(25 if max_length >= 25 else max_length)
            kwargs[username_field.name] = username

            # Set value for 'email' field in case the user model has one
            try:
                User._meta.get_field("email")
                kwargs["email"] = email
            except FieldDoesNotExist:
                pass

        user = User(**kwargs)

    if request.method == "POST":
        form = form_class(user, request.POST)
        if form.is_valid():
            user = form.save()

            password_set.send(
                sender=user.__class__,
                request=request,
                user=user,
                password=form.cleaned_data.get("new_password1"),
            )

            messages.success(
                request, _("Successfully set the new password. Please login now.")
            )

            return redirect("login")

    else:
        messages.success(request, _("Please set a password."))
        form = form_class(user)

    return render(
        request,
        "registration/password_set_form.html",
        {
            "form": form,
        },
    )



from django.contrib.auth.models import User
from django.core import signing
from django.core.mail import EmailMultiAlternatives
from django.template.loader import TemplateDoesNotExist, render_to_string
from django.utils.http import int_to_base36
from django.utils.translation import gettext as _


try:
    from django.urls import reverse
except ImportError:  # pragma: no cover
    from django.core.urlresolvers import reverse


def get_signer(salt="email_registration"):
    """
    Returns the signer instance used to sign and unsign the registration
    link tokens
    """
    return signing.TimestampSigner(salt=salt)


def get_last_login_timestamp(user):
    """
    Django 1.7 allows the `last_login` timestamp to be `None` for new users.
    """
    return int(user.last_login.strftime("%s")) if user.last_login else 0


def get_confirmation_url(email, request, user=None):
    """
    Returns the confirmation URL
    """
    code = [email, "", ""]
    if user:
        code[1] = str(user.id)
        code[2] = int_to_base36(get_last_login_timestamp(user))

    return request.build_absolute_uri(
        reverse(
            "email_registration_confirm",
            kwargs={
                "code": get_signer().sign(":".join(code)),
            },
        )
    )


def send_registration_mail(email, request, user=None):
    """
    Sends the registration mail

    * ``email``: The email address where the registration link should be
      sent to.
    * ``request``: A HTTP request instance, used to construct the complete
      URL (including protocol and domain) for the registration link.
    * ``user``: Optional user instance. If the user exists already and you
      just want to send a link where the user can choose his/her password.

    The mail is rendered using the following two templates:

    * ``registration/email_registration_email.txt``: The first line of this
      template will be the subject, the third to the last line the body of the
      email.
    * ``registration/email_registration_email.html``: The body of the HTML
      version of the mail. This template is **NOT** available by default and
      is not required either.
    """

    render_to_mail(
        "registration/email_registration_email",
        {
            "url": get_confirmation_url(email, request, user=user),
        },
        to=[email],
    ).send()


class InvalidCode(Exception):
    """Problems occurred during decoding the registration link"""

    pass


def decode(code, max_age=3 * 86400):
    """
    Decodes the code from the registration link and returns a tuple consisting
    of the verified email address and the associated user instance or ``None``
    if no user was passed to ``send_registration_mail``

    This method raises ``InvalidCode`` exceptions containing an translated
    message what went wrong suitable for presenting directly to the user.
    """
    try:
        data = get_signer().unsign(code, max_age=max_age)
    except signing.SignatureExpired:
        raise InvalidCode(
            _("The link is expired. Please request another registration link.")
        )

    except signing.BadSignature:
        raise InvalidCode(
            _(
                "Unable to verify the signature. Please request a new"
                " registration link."
            )
        )

    parts = data.rsplit(":", 2)
    if len(parts) != 3:
        raise InvalidCode(
            _(
                "Something went wrong while decoding the"
                " registration request. Please try again."
            )
        )

    email, uid, timestamp = parts
    if uid and timestamp:
        try:
            user = User.objects.get(pk=uid)
        except (User.DoesNotExist, TypeError, ValueError):
            raise InvalidCode(
                _(
                    "Something went wrong while decoding the"
                    " registration request. Please try again."
                )
            )

        if timestamp != int_to_base36(get_last_login_timestamp(user)):
            raise InvalidCode(_("The link has already been used."))

    else:
        user = None

    return email, user


def render_to_mail(template, context, **kwargs):
    """
    Renders a mail and returns the resulting ``EmailMultiAlternatives``
    instance

    * ``template``: The base name of the text and HTML (optional) version of
      the mail.
    * ``context``: The context used to render the mail. This context instance
      should contain everything required.
    * Additional keyword arguments are passed to the ``EmailMultiAlternatives``
      instantiation. Use those to specify the ``to``, ``headers`` etc.
      arguments.

    Usage example::

        # Render the template myproject/hello_mail.txt (first line contains
        # the subject, third to last the body) and optionally the template
        # myproject/hello_mail.html containing the alternative HTML
        # representation.
        message = render_to_mail('myproject/hello_mail', {}, to=[email])
        message.send()
    """
    lines = iter(render_to_string("%s.txt" % template, context).splitlines())

    subject = ""
    while True:
        line = next(lines)
        if line:
            subject = line
            break

    body = "\n".join(lines).strip("\n")
    message = EmailMultiAlternatives(subject=subject, body=body, **kwargs)

    try:
        message.attach_alternative(
            render_to_string("%s.html" % template, context), "text/html"
        )
    except TemplateDoesNotExist:
        pass

    return message



#!/usr/bin/env python

import os

from setuptools import find_packages, setup


def read(filename):
    return open(os.path.join(os.path.dirname(__file__), filename)).read()


setup(
    name="django-email-registration",
    version=__import__("email_registration").__version__,
    description="So simple you'll burst into tears right away.",
    long_description=read("README.rst"),
    author="Matthias Kestenholz",
    author_email="mk@406.ch",
    url="http://github.com/matthiask/django-email-registration/",
    license="BSD License",
    platforms=["OS Independent"],
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
    ],
    zip_safe=False,
)
